import { css, ReactiveElement } from 'lit';
import { ContextConsumer } from '@lit-labs/context';
import { extensionsContext, } from './extension-host-context.js';
import {} from '@codemirror/state';
class CodeMirrorExtensionElement extends ReactiveElement {
    static { this.styles = css `
    :host {
      display: none;
    }
  `; }
    /**
     * The extensions that this element adds to its nearest
     * CodeMirrorEditorElement.
     */
    #extensions;
    constructor() {
        super();
        new ContextConsumer(this, extensionsContext, (extensionHost) => {
            if (this.#extensions !== undefined &&
                this.extensionHost !== extensionHost) {
                this.extensionHost?.removeExtensions(this.#extensions);
                extensionHost?.addExtensions(this.#extensions);
            }
            this.extensionHost = extensionHost;
        }, true);
    }
    setExtensions(extensions) {
        if (this.#extensions !== undefined) {
            this.extensionHost?.removeExtensions(this.#extensions);
        }
        this.#extensions = extensions;
        if (this.#extensions !== undefined) {
            this.extensionHost?.addExtensions(this.#extensions);
        }
    }
    disconnectedCallback() {
        super.disconnectedCallback();
        if (this.#extensions !== undefined) {
            this.extensionHost?.removeExtensions(this.#extensions);
        }
        this.extensionHost = undefined;
    }
}
export { CodeMirrorExtensionElement };
//# sourceMappingURL=codemirror-extention-element.js.map